<?php
/**
 *
 * @author Jan-Hendrik Willms <tleilax@mindfuck.de>
 * @author Dennis Reil <dennis.reil@offis.de>
 * @author Nico Mueller <nico.mueller@uni-oldenburg.de>
 * @author Michael Riehemann <michael.riehemann@uni-oldenburg.de>
 * @author Eric Laubmeyer<laubmeyer@ite.fh-wiesbaden.de>
 * @version 2.1.0
 *
 */
require_once 'lib/include/reiter.inc.php';

class TipsUndTricksPlugin extends AbstractStudIPPortalPlugin
{
	public $user;
	private $permissions;
	private $db;
	private $template_factory;

	private $tabs = array(
		'admin'=>'bersicht',
		'create'=>'Neuen Tipp erstellen'
	);

	// user obj
	var $user_class = NULL;
	// user obj
	var $teacher_class = NULL;
	// init later
	var $return_link = '';
	// init later
	var $home_page_username = '';
	// error
	var $error_msg = "Eingabe NICHT erfolgreich &uuml;bernommen.<br>ERROR : ";
	// ok
	var $OK_message = "Eingabe erfolgreich &uuml;bernommen.";


	public function __construct()
	{
		if (get_magic_quotes_gpc() or get_magic_quotes_runtime())
		{
			if (!function_exists('strip_slashes_deep'))
			{
				function strip_slashes_deep($var)
				{
					return (is_array($var))
						? array_map('strip_slashes_deep', $var)
						: stripslashes($var);
				}
			}
    		$_POST = strip_slashes_deep($_POST);
    		$_GET = strip_slashes_deep($_GET);
    		$_COOKIE = strip_slashes_deep($_COOKIE);
		}

		parent::AbstractStudIPPortalPlugin();

		$nav = new PluginNavigation();
		$nav->setDisplayname($this->getPluginname());
		$this->setNavigation($nav);

		$this->user = $this->getUser();
		$this->permissions = $this->user->getPermission();
		$this->db = new DB_Seminar;
		$this->template_factory =& new Flexi_TemplateFactory(dirname(__FILE__).'/templates');

		$this->setPluginiconname('img/info.gif');
	}

	public function getPluginname()
	{
  		return _('Tipps und Tricks');
	}

	public function hasAdministration()
	{
	    $cuser = $this->getUser();
	    foreach((array)$cuser->getAssignedRoles() as $role){
	    	if($role->getRolename() == 'tips_und_tricks_admin') $is_admin = true;
	    }
	    $cperms = $cuser->getPermission();
	    if ($cperms->hasRootPermission() || $is_admin)
	    {
		return true;
	    }
	    else
	    {
		return false;
	    }

	}


	/**
	 *	Main entry method for showing the tip of the day
	 */
 	public function showOverview()
 	{
 	    	$template = $this->template_factory->open('tip');
 		$template->set_attribute('tip', $this->getTipOfTheDay());
 		echo $template->render();
 	}

	public function getAdminLink()
	{
	    return PluginEngine::getLink($this,array('kind' => 'admin'));
	}


	public function display_action($action)
	{
	    if($this->hasAdministration()){

	    	$kind=$_GET['kind'];
	    	if (in_array($kind,words('admin edit create')) ) {
	    	    PageLayout::addScript('prototype.js');
	    	    include 'lib/include/html_head.inc.php';
	    	    include 'lib/include/header.php';
	    	}

			switch ($kind)
			{
			case 'admin':
				$this->showAdministrationPage();
				break;
			case 'activate':
				$this->_actionactivate();
				break;
			case 'create':
				$this->_actioncreate();
				break;
			case 'delete':
			$this->_actiondelete();
				break;
			case 'edit':
				$this->_actionedit();
				break;
			case 'toggle':
				$this->_actiontoggleVisibility();
				break;
			default:
				echo "Gibts nicht!";
			}

			if (in_array($kind,words('admin edit create')) ) {
			    include 'lib/include/html_end.inc.php';
			}
	    } else {
	    	throw new Studip_AccessDeniedException(_("Keine Berechtigung."));
	    }
	}

	private function startContentTable($full_width=true, $width="70%"){
                if (!$full_width){
                        echo ("<table border=\"0\" width=\"$width\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#ffffff\">");
                }
                else {
                        echo ("<table border=\"0\" width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#ffffff\">");
                }
                ?>
                <tr>
                        <td height="5" colspan="3"></td>
                </tr>
                <tr>
                        <td width="5">
                        </td>
                        <td valign="top">
                                <table border="0" cellpadding="0" cellspacing="0" width="100%">
                                <tr>
                                        <td>
                <?php
        }

 	private function getTipOfTheDay()
 	{
 		$this->db->queryf("SELECT perms FROM auth_user_md5 WHERE user_id='%s'", $this->user->getUserid());
 		$permissions = $this->db->next_record() ? $this->db->f('perms') : 'autor';
 		$this->db->free();

		$this->db->queryf("SELECT content, url FROM plugin_tips WHERE active='yes' AND FIND_IN_SET('%s', visibility) ORDER BY RAND(DATE_FORMAT(NOW(), '%%Y%%m%%d')) LIMIT 0,1", $permissions);
		if (!$this->db->next_record())
		{
			return false;
		}

 		$tip = array(
 			'content'=>$this->db->f('content'),
 			'url'=>$this->db->f('url')
 		);
 		$this->db->free();

 		return $tip;
 	}


	/**
	 *	Main entry method for administration and visualization of all existing tips
	 */
	public function showAdministrationPage()
	{
		$this->displayHeader();

		$tips = array();
		$this->db->queryf("SELECT tip_id, content, url, active, visibility FROM plugin_tips ORDER BY create_date ASC");
		while ($this->db->next_record())
		{
			array_push($tips, array(
				'id'=>$this->db->f('tip_id'),
				'content'=>$this->db->f('content'),
				'url'=>$this->db->f('url'),
				'active'=>$this->db->f('active')=='yes',
				'visibility'=>array(
					'autor'=>in_array('autor', explode(',', $this->db->f('visibility'))),
					'tutor'=>in_array('tutor', explode(',', $this->db->f('visibility'))),
					'dozent'=>in_array('dozent', explode(',', $this->db->f('visibility'))),
					'admin'=>in_array('admin', explode(',', $this->db->f('visibility'))),
					'root'=>in_array('root', explode(',', $this->db->f('visibility'))),
				)
			));
		}
		$this->db->free();

		$template = $this->template_factory->open('admin_overview');
		$template->set_attribute('img_url', $this->getPluginUrl() . '/img/');
		$template->set_attribute('tips', $tips);
		$template->set_attribute('links', array(
			'activate'=>PluginEngine::getLink($this, array('kind' => 'activate')),
			'delete'=>PluginEngine::getLink($this, array('kind' => 'delete')),
			'edit'=>PluginEngine::getLink($this, array('kind' => 'edit')),
			'toggle_visibility'=>PluginEngine::getLink($this, array('kind' => 'toggle'))
		));
		echo $template->render();
	}

	public function _actionedit()
	{
	    $this->displayHeader();

		$tip_id = (int)$_REQUEST['tip_id'];

		if (!empty($_POST))
		{
			$this->db->queryf("UPDATE plugin_tips SET content='%s', url=%s, visibility='%s', active='%s' WHERE tip_id=%d LIMIT 1",
				mysql_escape_string($_POST['content']),
				empty($_POST['url']) ? 'NULL' : "'".trim($_POST['url'])."'",
				implode(',', $_POST['visibility']),
				empty($_POST['active']) ? 'no' : 'yes',
				$tip_id
			);
			Header('Location: '."?kind=admin");
			die;

		}

		$this->db->queryf("SELECT content, url, visibility, active FROM plugin_tips WHERE tip_id=%d LIMIT 0,1", $tip_id);
		if (!$this->db->next_record())
		{
			throw new Exception('No data for tip '.$tip_id);
		}
		$tip = array(
			'id'=>$tip_id,
			'content'=>$this->db->f('content'),
			'url'=>$this->db->f('url'),
			'visibility'=>array(
				'autor'=>in_array('autor', explode(',', $this->db->f('visibility'))),
				'tutor'=>in_array('tutor', explode(',', $this->db->f('visibility'))),
				'dozent'=>in_array('dozent', explode(',', $this->db->f('visibility'))),
				'admin'=>in_array('admin', explode(',', $this->db->f('visibility'))),
				'root'=>in_array('root', explode(',', $this->db->f('visibility'))),
			),
			'active'=>$this->db->f('active')=='yes'
		);

		$template = $this->template_factory->open('admin_edit');
		$template->set_attribute('tip', $tip);
		$template->set_attribute('links', array(
			'submit'=>PluginEngine::getLink($this, array('kind' => 'edit', 'tip_id' => $tip_id))));
		echo $template->render();
	}

	public function _actioncreate()
	{
		$this->displayHeader();

		if (!empty($_POST))
		{
			$this->db->queryf("INSERT INTO plugin_tips (content, url, visibility, active, create_date) VALUES (TRIM('%s'), %s, '%s', '%s', NOW())",
				mysql_escape_string($_POST['content']),
				empty($_POST['url']) ? 'NULL' : "'".trim($_POST['url'])."'",
				implode(',', $_POST['visibility']),
				empty($_POST['active']) ? 'no' : 'yes'
			);
			if ($this->db->affected_rows()!=-1)
			{
				$this->db->queryf("SELECT LAST_INSERT_ID() AS tip_id");
				$tip_id = $this->db->next_record() ? $this->db->f('tip_id') : '';
				$this->db->free();

				Header('Location: '."?kind=admin");
				die;
			}
		}

		$template = $this->template_factory->open('admin_create');
		$template->set_attribute('links', array(
			'submit'=>PluginEngine::getLink($this, array('kind' => 'create', 'tip_id' => $tip_id))
		));
		echo $template->render();
	}

	public function displayHeader()
	{
		$active = $_GET['kind'];

		$structure = array();
		$first = true;
		foreach ($this->tabs as $cmd=>$title)
		{
			$structure[$cmd] = array(
				'topKat'=>'',
				'name'=>_($title),
				'link'=>PluginEngine::getLink($this, array('kind' => $cmd)),
				'active'=>''
			);
			$first = false;
		}

		$reiter = new reiter;
		$reiter->create($structure, $active);

		$this->startContentTable(true);

		echo '<br/>';
	}

	public function _actionactivate()
	{
		$tip_id = $_GET['tip_id'];
		$this->db->queryf("UPDATE plugin_tips SET active=(SELECT CASE active WHEN 'yes' THEN 'no' ELSE 'yes' END) WHERE tip_id=%d LIMIT 1", $tip_id);

		Header('Location: '."?kind=admin");
		die;
	}

	public function _actiontoggleVisibility()
	{
		$key = $_GET['key'];
		$tip_id = $_GET['tip_id'];

		$this->db->queryf("UPDATE plugin_tips SET visibility =(SELECT CASE LOCATE('%s', visibility) WHEN 0 THEN CONCAT_WS(',', visibility, '%s') ELSE REPLACE(visibility, '%s', '') END) WHERE tip_id=%d LIMIT 1", $key, $key, $key, $tip_id);

		Header('Location: '."?kind=admin");
		die;
	}

	public function _actiondelete()
	{
		$tip_id = $_GET['tip_id'];
		$this->db->queryf("DELETE FROM plugin_tips WHERE tip_id=%d LIMIT 1", $tip_id);

		Header('Location: '."?kind=admin");
		die;
	}
}

?>
